@@ -301,11 +301,13 @@ class PostgreSQLExtractor(BaseExtractor):
 
         return SchemaField(
             field_path=column["column_name"],
+            field_name=column["column_name"],
             native_type=native_type,
             nullable=(not column["not_null"]),
             description=column["description"],
             max_length=max_length,
             precision=precision,
+            subfields=None,
         )
 
     @staticmethod
